import React, { useCallback, useContext, useEffect, useRef, useState } from "react";
import { BrowserProvider } from "ethers";
import { SIWESigner } from "@canvas-js/signer-ethereum";
import { AuthContext } from "../AuthContext.js";
import { styles } from "./styles.js";
export const useSIWE = (app) => {
    const { sessionSigner, setSessionSigner, address, setAddress } = useContext(AuthContext);
    const [provider, setProvider] = useState(null);
    const [error, setError] = useState(null);
    const connect = useCallback(async (provider, isBrowserInit) => {
        if (!app) {
            setError(new Error("app not initialized"));
            return;
        }
        if (provider === null) {
            setError(new Error("window.ethereum not found"));
            return;
        }
        setProvider(provider);
        const network = await provider.getNetwork();
        const signer = await provider
            .getSigner()
            .then((signer) => new SIWESigner({ signer, chainId: Number(network.chainId) }));
        if (!(await signer.hasSession(app.topic))) {
            if (isBrowserInit) {
                return;
            }
            else {
                const session = await signer.newSession(app.topic);
                await app.messageLog.append(session.payload, { signer: session.signer });
            }
        }
        const address = await signer.getDid();
        setAddress(address);
        const otherSigners = app.signers.getAll().filter((s) => !(s instanceof SIWESigner));
        otherSigners.forEach((s) => s.clearSession(app.topic));
        app.updateSigners([signer, ...otherSigners]);
        setSessionSigner(signer);
    }, [app, provider]);
    const initialRef = useRef(false);
    useEffect(() => {
        if (initialRef.current || !app) {
            return;
        }
        initialRef.current = true;
        if (window.ethereum === undefined || window.ethereum === null) {
            setError(new Error("window.ethereum not found"));
            return;
        }
        // TODO: handle these more gracefully
        window.ethereum.on("chainChanged", (chainId) => window.location.reload());
        window.ethereum.on("accountsChanged", (accounts) => window.location.reload());
        const provider = new BrowserProvider(window.ethereum);
        setProvider(provider);
        // automatically log back in
        connect(provider, true);
    }, [app]);
    const disconnect = useCallback(async () => {
        if (!app?.topic)
            return;
        sessionSigner?.clearSession(app.topic);
        setAddress(null);
        setSessionSigner(null);
        const otherSigners = app.signers.getAll().filter((s) => !(s instanceof SIWESigner));
        app.updateSigners([...otherSigners, new SIWESigner()]);
    }, [app, sessionSigner]);
    const ConnectSIWE = ({ buttonStyles, errorStyles, errorClassName, buttonClassName, } = {}) => {
        if (!app) {
            return (React.createElement("div", { className: errorClassName || "", style: {
                    ...styles.errorContainer,
                    ...errorStyles
                } },
                React.createElement("code", null, "App not initialized")));
        }
        else if (error !== null) {
            return (React.createElement("div", { className: errorClassName || "", style: {
                    ...styles.errorContainer,
                    ...errorStyles
                } },
                React.createElement("code", null, error.message)));
        }
        else if (provider === null) {
            return (React.createElement("button", { disabled: true, className: buttonClassName || "", style: {
                    ...styles.loadingButton,
                    ...buttonStyles
                } }, "Loading..."));
        }
        else if (address !== null && sessionSigner instanceof SIWESigner) {
            return (React.createElement("button", { onClick: () => disconnect(), className: buttonClassName || "", style: {
                    ...styles.actionButton,
                    ...buttonStyles
                } }, "Disconnect ETH wallet"));
        }
        else {
            return (React.createElement("button", { onClick: () => {
                    connect(new BrowserProvider(window.ethereum));
                }, className: buttonClassName || "", style: {
                    ...styles.actionButton,
                    ...buttonStyles
                } }, "Connect ETH wallet"));
        }
    };
    return {
        ConnectSIWE,
        address,
        error,
        provider,
        connect,
        disconnect,
        sessionSigner,
        isInitialized: !!app,
    };
};
